@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2009-2011 Nokia Corporation and/or its subsidiary(-ies).
+ *  Copyright (C) 2009-2011  Nokia Corporation and/or its subsidiary(-ies).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,7 +37,6 @@
 #include <ofono/modem.h>
 #include <ofono/call-settings.h>
 
-#include "isimodem.h"
 #include "isiutil.h"
 #include "ss.h"
 #include "debug.h"
@@ -405,25 +403,14 @@ static void isi_call_settings_remove(struct ofono_call_settings *cs)
 	g_free(data);
 }
 
-static struct ofono_call_settings_driver driver = {
-	.name			= "isimodem",
+static const struct ofono_call_settings_driver driver = {
 	.probe			= isi_call_settings_probe,
 	.remove			= isi_call_settings_remove,
 	.clip_query		= isi_clip_query,
 	.colp_query		= isi_colp_query,
 	.colr_query		= isi_colr_query,
-	.clir_query		= NULL,
-	.clir_set		= NULL,
 	.cw_query		= isi_cw_query,
 	.cw_set			= isi_cw_set
 };
 
-void isi_call_settings_init(void)
-{
-	ofono_call_settings_driver_register(&driver);
-}
-
-void isi_call_settings_exit(void)
-{
-	ofono_call_settings_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(call_settings, isimodem, &driver)
