@@ -30,6 +30,7 @@
 #include <glib.h>
 
 #include <ofono/types.h>
+#include <ofono/gprs-context.h>
 #include "common.h"
 #include "util.h"
 
@@ -49,13 +50,13 @@ struct error_entry cms_errors[] = {
 	{ 10,	"Call barred" },
 	{ 21,	"Short message transfer rejected" },
 	{ 27,	"Destination out of service" },
-	{ 28,	"Unindentified subscriber" },
+	{ 28,	"Unidentified subscriber" },
 	{ 29,	"Facility rejected" },
 	{ 30,	"Unknown subscriber" },
 	{ 38,	"Network out of order" },
 	{ 41,	"Temporary failure" },
 	{ 42,	"Congestion" },
-	{ 47,	"Recources unavailable" },
+	{ 47,	"Resources unavailable" },
 	{ 50,	"Requested facility not subscribed" },
 	{ 69,	"Requested facility not implemented" },
 	{ 81,	"Invalid short message transfer reference value" },
@@ -65,8 +66,8 @@ struct error_entry cms_errors[] = {
 	{ 98,	"Message not compatible with short message protocol state" },
 	{ 99,	"Information element non-existent or not implemented" },
 	{ 111,	"Protocol error, unspecified" },
-	{ 127,	"Internetworking error, unspecified" },
-	{ 128,	"Telematic internetworking not supported" },
+	{ 127,	"Interworking error, unspecified" },
+	{ 128,	"Telematic interworking not supported" },
 	{ 129,	"Short message type 0 not supported" },
 	{ 130,	"Cannot replace short message" },
 	{ 143,	"Unspecified TP-PID error" },
@@ -89,7 +90,7 @@ struct error_entry cms_errors[] = {
 	{ 209,	"No SMS Storage capability in SIM" },
 	{ 210,	"Error in MS" },
 	{ 211,	"Memory capacity exceeded" },
-	{ 212,	"Sim application toolkit busy" },
+	{ 212,	"SIM application toolkit busy" },
 	{ 213,	"SIM data download error" },
 	{ 255,	"Unspecified error cause" },
 	{ 300,	"ME Failure" },
@@ -121,7 +122,7 @@ struct error_entry cms_errors[] = {
 struct error_entry cme_errors[] = {
 	{ 0,	"Phone failure" },
 	{ 1,	"No connection to phone" },
-	{ 2,	"Phone adapter link reserved" },
+	{ 2,	"Phone adaptor link reserved" },
 	{ 3,	"Operation not allowed" },
 	{ 4,	"Operation not supported" },
 	{ 5,	"PH_SIM PIN required" },
@@ -156,6 +157,7 @@ struct error_entry cme_errors[] = {
 	{ 46,	"Corporate personalization PIN required" },
 	{ 47,	"Corporate personalization PUK required" },
 	{ 48,	"PH-SIM PUK required" },
+	{ 50,	"Incorrect parameters" },
 	{ 100,	"Unknown error" },
 	{ 103,	"Illegal MS" },
 	{ 106,	"Illegal ME" },
@@ -204,14 +206,14 @@ struct error_entry ceer_errors[] = {
 	{ 38,	"Network out of order" },
 	{ 41,	"Temporary failure" },
 	{ 42,	"Switching equipment congestion" },
-	{ 43,	"Access information discared" },
+	{ 43,	"Access information discarded" },
 	{ 44,	"Requested circuit/channel not available" },
 	{ 47,	"Resource unavailable (unspecified)" },
 	{ 49,	"Quality of service unavailable" },
 	{ 50,	"Requested facility not subscribed" },
 	{ 55,	"Incoming calls barred within the CUG" },
 	{ 57,	"Bearer capability not authorized" },
-	{ 58,	"Bearar capability not presently available" },
+	{ 58,	"Bearer capability not presently available" },
 	{ 63,	"Service or option not available, unspecified" },
 	{ 65,	"Bearer service not implemented" },
 	{ 68,	"ACM equal to or greater than ACMmax" },
@@ -229,7 +231,7 @@ struct error_entry ceer_errors[] = {
 	{ 99,	"Information element non-existent or not implemented" },
 	{ 100,	"Conditional IE error" },
 	{ 101,	"Message not compatible with protocol state" },
-	{ 102,	"Recovery on timer expirty" },
+	{ 102,	"Recovery on timer expiry" },
 	{ 111,	"Protocol error, unspecified" },
 	{ 127,	"Interworking, unspecified" },
 };
@@ -281,30 +283,6 @@ gboolean valid_long_phone_number_format(const char *number)
 	return valid_number_format(number, OFONO_MAX_PHONE_NUMBER_LENGTH);
 }
 
-gboolean valid_cdma_phone_number_format(const char *number)
-{
-	int len = strlen(number);
-	int i;
-
-	if (!len)
-		return FALSE;
-
-	if (len > OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH)
-		return FALSE;
-
-	for (i = 0; i < len; i++) {
-		if (number[i] >= '0' && number[i] <= '9')
-			continue;
-
-		if (number[i] == '*' || number[i] == '#')
-			continue;
-
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 const char *telephony_error_to_str(const struct ofono_error *error)
 {
 	struct error_entry *e;
@@ -448,23 +426,6 @@ void string_to_phone_number(const char *str, struct ofono_phone_number *ph)
 	}
 }
 
-const char *cdma_phone_number_to_string(
-				const struct ofono_cdma_phone_number *ph)
-{
-	static char buffer[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH + 1];
-
-	strncpy(buffer, ph->number, OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH);
-	buffer[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH] = '\0';
-
-	return buffer;
-}
-
-void string_to_cdma_phone_number(const char *str,
-					struct ofono_cdma_phone_number *ph)
-{
-	strcpy(ph->number, str);
-}
-
 gboolean valid_ussd_string(const char *str, gboolean call_in_progress)
 {
 	int len = strlen(str);
@@ -491,7 +452,7 @@ gboolean valid_ussd_string(const char *str, gboolean call_in_progress)
 	if (str[len-1] == '#')
 		return TRUE;
 
-	if (!call_in_progress && len == 2 && str[0] != '1')
+	if (!call_in_progress && len == 2 && str[0] == '1')
 		return FALSE;
 
 	if (len <= 2)
@@ -553,7 +514,7 @@ gboolean parse_ss_control_string(char *str, int *ss_type,
 
 	cur = 1;
 
-	if (str[1] != '*' && str[1] != '#' && str[1] > '9' && str[1] < '0')
+	if (str[1] != '*' && str[1] != '#' && (str[1] > '9' || str[1] < '0'))
 		goto out;
 
 	if (str[0] == '#' && str[1] == '*')
@@ -667,6 +628,10 @@ const char *registration_status_to_string(int status)
 		return "unknown";
 	case NETWORK_REGISTRATION_STATUS_ROAMING:
 		return "roaming";
+	case NETWORK_REGISTRATION_STATUS_REGISTERED_SMS_EUTRAN:
+		return "registered";
+	case NETWORK_REGISTRATION_STATUS_ROAMING_SMS_EUTRAN:
+		return "roaming";
 	}
 
 	return "";
@@ -691,6 +656,10 @@ const char *registration_tech_to_string(int tech)
 		return "hspa";
 	case ACCESS_TECHNOLOGY_EUTRAN:
 		return "lte";
+	case ACCESS_TECHNOLOGY_NB_IOT_M1:
+		return "lte-cat-m1";
+	case ACCESS_TECHNOLOGY_NB_IOT_NB1:
+		return "lte-cat-nb1";
 	default:
 		return "";
 	}
@@ -699,11 +668,17 @@ const char *registration_tech_to_string(int tech)
 gboolean is_valid_apn(const char *apn)
 {
 	int i;
-	int last_period = 0;
+	int last_period = -1;
+
+	if (apn == NULL)
+		return FALSE;
 
 	if (apn[0] == '.' || apn[0] == '\0')
 		return FALSE;
 
+	if (strlen(apn) > OFONO_GPRS_MAX_APN_LENGTH)
+		return FALSE;
+
 	for (i = 0; apn[i] != '\0'; i++) {
 		if (g_ascii_isalnum(apn[i]))
 			continue;
@@ -735,3 +710,86 @@ void ofono_call_init(struct ofono_call *call)
 	call->cnap_validity = CNAP_VALIDITY_NOT_AVAILABLE;
 	call->clip_validity = CLIP_VALIDITY_NOT_AVAILABLE;
 }
+
+const char *call_status_to_string(enum call_status status)
+{
+	switch (status) {
+	case CALL_STATUS_ACTIVE:
+		return "active";
+	case CALL_STATUS_HELD:
+		return "held";
+	case CALL_STATUS_DIALING:
+		return "dialing";
+	case CALL_STATUS_ALERTING:
+		return "alerting";
+	case CALL_STATUS_INCOMING:
+		return "incoming";
+	case CALL_STATUS_WAITING:
+		return "waiting";
+	case CALL_STATUS_DISCONNECTED:
+		return "disconnected";
+	}
+
+	return "unknown";
+}
+
+const char *gprs_proto_to_string(enum ofono_gprs_proto proto)
+{
+	switch (proto) {
+	case OFONO_GPRS_PROTO_IP:
+		return "ip";
+	case OFONO_GPRS_PROTO_IPV6:
+		return "ipv6";
+	case OFONO_GPRS_PROTO_IPV4V6:
+		return "dual";
+	};
+
+	return NULL;
+}
+
+gboolean gprs_proto_from_string(const char *str, enum ofono_gprs_proto *proto)
+{
+	if (g_str_equal(str, "ip")) {
+		*proto = OFONO_GPRS_PROTO_IP;
+		return TRUE;
+	} else if (g_str_equal(str, "ipv6")) {
+		*proto = OFONO_GPRS_PROTO_IPV6;
+		return TRUE;
+	} else if (g_str_equal(str, "dual")) {
+		*proto = OFONO_GPRS_PROTO_IPV4V6;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+const char *gprs_auth_method_to_string(enum ofono_gprs_auth_method auth)
+{
+	switch (auth) {
+	case OFONO_GPRS_AUTH_METHOD_CHAP:
+		return "chap";
+	case OFONO_GPRS_AUTH_METHOD_PAP:
+		return "pap";
+	case OFONO_GPRS_AUTH_METHOD_NONE:
+		return "none";
+	};
+
+	return NULL;
+}
+
+gboolean gprs_auth_method_from_string(const char *str,
+					enum ofono_gprs_auth_method *auth)
+{
+	if (g_str_equal(str, "chap")) {
+		*auth = OFONO_GPRS_AUTH_METHOD_CHAP;
+		return TRUE;
+	} else if (g_str_equal(str, "pap")) {
+		*auth = OFONO_GPRS_AUTH_METHOD_PAP;
+		return TRUE;
+	} else if (g_str_equal(str, "none")) {
+		*auth = OFONO_GPRS_AUTH_METHOD_NONE;
+		return TRUE;
+	}
+
+	return FALSE;
+}
