@@ -31,18 +31,19 @@
 #include <gdbus.h>
 
 #include "ofono.h"
-
 #include "common.h"
+#include "missing.h"
 
-static GSList *g_devinfo_drivers = NULL;
-static GSList *g_driver_list = NULL;
-static GSList *g_modem_list = NULL;
+#define DEFAULT_POWERED_TIMEOUT (20)
 
-static int next_modem_id = 0;
-static gboolean powering_down = FALSE;
-static int modems_remaining = 0;
+static GSList *g_driver_list;
+static GSList *g_modem_list;
 
-static struct ofono_watchlist *g_modemwatches = NULL;
+static int next_modem_id;
+static gboolean powering_down;
+static int modems_remaining;
+
+static struct ofono_watchlist *g_modemwatches;
 
 enum property_type {
 	PROPERTY_TYPE_INVALID = 0,
@@ -75,6 +76,7 @@ struct ofono_modem {
 	char			*lock_owner;
 	guint			lock_watch;
 	guint			timeout;
+	guint			timeout_hint;
 	ofono_bool_t		online;
 	struct ofono_watchlist	*online_watches;
 	struct ofono_watchlist	*powered_watches;
@@ -454,6 +456,28 @@ static void flush_atoms(struct ofono_modem *modem, enum modem_state new_state)
 	}
 }
 
+const void *__ofono_atom_driver_builtin_find(const char *name,
+				const struct ofono_atom_driver_desc *start,
+				const struct ofono_atom_driver_desc *stop)
+{
+	const struct ofono_atom_driver_desc *desc;
+
+	if (!name)
+		return NULL;
+
+	for (desc = start; desc < stop; desc++) {
+		if (!desc->name || !desc->driver)
+			continue;
+
+		if (strcmp(desc->name, name))
+			continue;
+
+		return desc->driver;
+	}
+
+	return NULL;
+}
+
 static void notify_online_watches(struct ofono_modem *modem)
 {
 	struct ofono_watchlist_item *item;
@@ -1055,7 +1079,7 @@ static DBusMessage *set_property_lockdown(struct ofono_modem *modem,
 		}
 
 		modem->pending = dbus_message_ref(msg);
-		modem->timeout = g_timeout_add_seconds(20,
+		modem->timeout = g_timeout_add_seconds(modem->timeout_hint,
 						set_powered_timeout, modem);
 		return NULL;
 	}
@@ -1127,13 +1151,17 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 		if (modem->lockdown)
 			return __ofono_error_access_denied(msg);
 
+		if (!powered)
+			__ofono_sim_clear_cached_pins(modem->sim);
+
 		err = set_powered(modem, powered);
 		if (err < 0) {
 			if (err != -EINPROGRESS)
 				return __ofono_error_failed(msg);
 
 			modem->pending = dbus_message_ref(msg);
-			modem->timeout = g_timeout_add_seconds(20,
+			modem->timeout = g_timeout_add_seconds(
+						modem->timeout_hint,
 						set_powered_timeout, modem);
 			return NULL;
 		}
@@ -1596,25 +1624,6 @@ static void dun_watch(struct ofono_atom *atom,
 	ofono_emulator_add_handler(em, "+GCAP", gcap_cb, data, NULL);
 }
 
-int ofono_devinfo_driver_register(const struct ofono_devinfo_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_devinfo_drivers = g_slist_prepend(g_devinfo_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_devinfo_driver_unregister(const struct ofono_devinfo_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_devinfo_drivers = g_slist_remove(g_devinfo_drivers, (void *) d);
-}
-
 static void devinfo_remove(struct ofono_atom *atom)
 {
 	struct ofono_devinfo *info = __ofono_atom_get_data(atom);
@@ -1632,34 +1641,7 @@ static void devinfo_remove(struct ofono_atom *atom)
 	g_free(info);
 }
 
-struct ofono_devinfo *ofono_devinfo_create(struct ofono_modem *modem,
-							unsigned int vendor,
-							const char *driver,
-							void *data)
-{
-	struct ofono_devinfo *info;
-	GSList *l;
-
-	info = g_new0(struct ofono_devinfo, 1);
-
-	info->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_DEVINFO,
-						devinfo_remove, info);
-
-	for (l = g_devinfo_drivers; l; l = l->next) {
-		const struct ofono_devinfo_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(info, vendor, data) < 0)
-			continue;
-
-		info->driver = drv;
-		break;
-	}
-
-	return info;
-}
+OFONO_DEFINE_ATOM_CREATE(devinfo, OFONO_ATOM_TYPE_DEVINFO)
 
 static void devinfo_unregister(struct ofono_atom *atom)
 {
@@ -1747,10 +1729,10 @@ static int set_modem_property(struct ofono_modem *modem, const char *name,
 		property->value = g_strdup((const char *) value);
 		break;
 	case PROPERTY_TYPE_INTEGER:
-		property->value = g_memdup(value, sizeof(int));
+		property->value = g_memdup2(value, sizeof(int));
 		break;
 	case PROPERTY_TYPE_BOOLEAN:
-		property->value = g_memdup(value, sizeof(ofono_bool_t));
+		property->value = g_memdup2(value, sizeof(ofono_bool_t));
 		break;
 	default:
 		break;
@@ -1843,6 +1825,12 @@ ofono_bool_t ofono_modem_get_boolean(struct ofono_modem *modem, const char *key)
 	return value;
 }
 
+void ofono_modem_set_powered_timeout_hint(struct ofono_modem *modem,
+							unsigned int seconds)
+{
+	modem->timeout_hint = seconds;
+}
+
 void ofono_modem_set_name(struct ofono_modem *modem, const char *name)
 {
 	if (modem->name)
@@ -1904,6 +1892,7 @@ struct ofono_modem *ofono_modem_create(const char *name, const char *type)
 	modem->driver_type = g_strdup(type);
 	modem->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, unregister_property);
+	modem->timeout_hint = DEFAULT_POWERED_TIMEOUT;
 
 	g_modem_list = g_slist_prepend(g_modem_list, modem);
 
